<?php

include_once(dirname(__FILE__).'/defines/defines.inc.php');
include_once(dirname(__FILE__).'/Export/EditorHelper.php');
include_once(dirname(__FILE__).'/Export/FilesHelper.php');

class Themler extends Module
{
    public function __construct()
    {
		$this->name = 'themler';
        $this->tab = 'front_office_features';
        $this->version = '2.0.0';
        $this->author = 'Themler';
        $this->controllers = array('AdminAjax');
        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->trans('Themler');
        $this->description = $this->trans('This module must be enabled if you want to use Themler.');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);        	
    }

    public function install()
    {
        if(!parent::install()
		 || !$this->Register_Tabs()
         || !$this->registerHook('displayNavigationBar')
		)
			return false;
		return true;
    }
	
    private function Register_Tabs()
    {
		$tabs_lists = array();
        $langs = Language::getLanguages();
        $id_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        $save_tab_id = $this->Register_ETabs();
    	if(isset($this->all_tabs) && !empty($this->all_tabs)){
    		foreach ($this->all_tabs as $tab_list)
    		{
    		    $tab_listobj = new Tab();
    		    $tab_listobj->class_name = $tab_list['class_name'];
    		    $tab_listobj->id_parent = $save_tab_id;
    		    if(isset($tab_list['module']) && !empty($tab_list['module'])){
    		    	$tab_listobj->module = $tab_list['module'];
    		    }else{
    		    	$tab_listobj->module = $this->name;
    		    }
    		    foreach($langs as $l)
    		    {
    		    	$tab_listobj->name[$l['id_lang']] = $this->l($tab_list['name']);
    		    }
    		    $tab_listobj->save();
    		}
    	}
        return true;
    }

	public function Register_ETabs(){
		$tabpar_listobj = new Tab();
		$langs = Language::getLanguages();
		$id_parent = (int)Tab::getIdFromClassName("IMPROVE");
		$tabpar_listobj->class_name = 'AdminAjax';
		$tabpar_listobj->id_parent = $id_parent;
		$tabpar_listobj->module = $this->name;
		foreach($langs as $l)
	    {
	    	$tabpar_listobj->name[$l['id_lang']] = "Themler";//$this->l("Themler");
	    }
	    if($tabpar_listobj->save()){
	    	return (int)$tabpar_listobj->id;
	    }else{
	    	return (int)$id_parent;
	    }
	}
	
    public function uninstall()
    {
        if(!parent::uninstall()
		 || !$this->UnRegister_Tabs()
         || !$this->unregisterHook('displayNavigationBar')
		)
			return false;
		return true;
    }
    
    public function UnRegister_Tabs()
	{
		if(isset($this->all_tabs) && !empty($this->all_tabs)){
			foreach($this->all_tabs as $tab_list){
				$tab_list_id = Tab::getIdFromClassName($tab_list['class_name']);
			    if(isset($tab_list_id) && !empty($tab_list_id)){
			        $tabobj = new Tab($tab_list_id);
			        $tabobj->delete();
			    }
			}
		}
		$tabp_list_id = Tab::getIdFromClassName('AdminAjax');
		$tabpobj = new Tab($tabp_list_id);
	    $tabpobj->delete();
        return true;
	}

    public static function getToken()
    {
        $context = Context::getContext();
        return Tools::getAdminToken('AdminAjax'.(int)Tab::getIdFromClassName('AdminAjax').(int)$context->employee->id);
    }

    public function getContent()
    {
        $token = Themler::getToken();
        header('Location: index.php?controller=AdminAjax&token=' . $token);
        exit;
    }
    
    public static function getEmployeeById($id_employee)
    {
        if (!Validate::isUnsignedId($id_employee))
            die (Tools::displayError());

        $result = Db::getInstance()->getRow('
        SELECT *
        FROM `'._DB_PREFIX_.'employee`
        WHERE `active` = 1
        AND `id_employee` = '.(int)$id_employee);
        if (!$result)
            return false;
        
        $employee = new Employee();
        $employee->id = $result['id_employee'];
        $employee->id_profile = $result['id_profile'];
        foreach ($result as $key => $value)
            if (property_exists($employee, $key))
                $employee->{$key} = $value;
        return $employee;
    }

    public static function getFirstItem($name)
    {
        $sql = 'SELECT MIN(`id_' . $name . '`) AS id_' . $name . ' FROM `'._DB_PREFIX_.$name.'`';
        if ($name == 'category') {
            $shop = self::getShop();
            $default_category = $shop->getCategory();
            $sql .= ' WHERE `id_parent` = '.(int)$default_category;
        }

        if ($id_child = Db::getInstance()->getValue($sql))
            return (int)$id_child;
        return 0;
    }
    
    public static function getBaseUrlWithLastSlash()
    {
        $baseUrl = (Tools::usingSecureMode() ? _PS_BASE_URL_SSL_ : _PS_BASE_URL_) . __PS_BASE_URI__;
        return $baseUrl . (substr($baseUrl, -1) !== '/' ? '/' : '');
    }
    
    public static function getDownloadUrl()
    {
        return self::getBaseUrlWithLastSlash() . 'download';
    }
    
    /*
     * Method generates a random 32 character string consisting of, by default, only A-Z, a-z, and 0-9
     */
    public static function generateThemeName()
    {
        $pattern = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        $length = 10; // themeName length
        for ($s = '', $i = 0, $z = strlen($pattern)-1; $i < $length; $x = rand(0,$z), $s .= $pattern{$x}, $i++);
        return $s;
    }

    public static function getShop()
    {
        $context = Context::getContext();
        return $context->shop;
    }

    public static function isActiveTheme($themeName)
    {
        $id_theme = self::getThemeId($themeName);
        if (!$id_theme)
            return false;

        $shop = self::getShop();
        return $shop->id_theme == $id_theme;
    }

    public static function setActiveThemeForShop($themeName)
    {
        $id_theme = self::getThemeId($themeName);
        if (!$id_theme) 
            $id_theme = self::addTheme($themeName);
        
        $shop = self::getShop();
        if ($id_theme && $shop->id_theme != $id_theme)
        {
            $shop->id_theme = $id_theme;
            $shop->update();
        }
        
        return $id_theme;
    }
    
    /*
     * Retrieves theme ID by name from database
     */
    public static function getThemeId($themeDir)
    {
        $result = 0;
        $sql = 'SELECT id_theme FROM `'._DB_PREFIX_.'theme`
            WHERE `directory` = \''.(string)$themeDir .'\'';
        try {
            if ($id_theme = Db::getInstance()->getValue($sql))
                $result = (int)$id_theme;
        } catch (Exception $e) {
            $result = 0;
        }
        return $result;
    }
    
    /*
     * Retrieves theme ID by name from database
     */
    public static function addTheme($themeName)
    {
        $sql = 'INSERT INTO `'._DB_PREFIX_.'theme` (`name`, `directory`) VALUES (\''.$themeName.'\', \''.$themeName.'\')';
        Db::getInstance()->execute($sql);
        return Db::getInstance()->Insert_ID();
    }
    
    /*
     * Retrieves theme ID by name from database
     */
    public static function deleteTheme($themeName)
    {
        $sql = 'DELETE FROM `'._DB_PREFIX_.'theme` WHERE name = \''.$themeName.'\'';
        if (!Db::getInstance()->execute($sql))
            return false;
    }

    public static function buildThemesData($currentIndex = '', $adminDir = '')
    {
        $token = self::getToken();
        $baseUrl = self::getBaseUrlWithLastSlash();
        $themesUrl = $baseUrl . 'themes';
        $adminUrl = $baseUrl . $adminDir . '/' . ($currentIndex ? $currentIndex : 'index.php?controller=AdminAjax') . '&token=' . $token;
        $shop = self::getShop();

		$context = Context::getContext();
        $db = Db::getInstance();
		$themeRepository = (new PrestaShop\PrestaShop\Core\Addon\Theme\ThemeManagerBuilder($context, $db))->buildRepository($shop);
		
        $themes = array();
        foreach ($themeRepository->getList() as $theme)
        {
			$themeName = $theme->getName();
			error_log($themeName );
            if (strpos($themeName, _PREVIEW_SUFFIX_) !== FALSE ||
                !file_exists(getProjectPath($themeName))) continue;

            $id = self::getThemeId($themeName);
            $themeUrl = $themesUrl . '/' . $themeName;
            $url = $adminUrl . '&theme_name=' . $themeName;
            $themes['themes'][$themeName] = array(
                'themeName' => $themeName,
                'thumbnailUrl' => $themeUrl. '/preview.png',
                'openUrl' => $url . '&ajax=1&ver=' . theme_get_manifest_version($themeName) . getDesktopParams(),
                'settingsUrl' => $url . '&edit=1',
                'isActive' => $id && $shop->id_theme == $id,
            );
        }
        if (isset($themes['themes']))
            return $themes;
        return 0;
    }
}